==> bosesoundtouchapi-1.0.20-py3-none-any.whl: bosesoundtouchapi/models/sourcelist.py <==
# external package imports.
from typing import Iterator
from xml.etree.ElementTree import Element, tostring
import xmltodict

# our package imports.
from ..bstutils import export, _xmlFind, _xmlFindAttr
from .sourceitem import SourceItem

@export
class SourceList:
    """
    SoundTouch device SourceList configuration object.
       
    This class contains the attributes and sub-items that represent the
    sources configuration of the device.
    """

    def __init__(self, root:Element=None) -> None:
        """
        Initializes a new instance of the class.
        
        Args:
            root (Element):
                xmltree Element item to load arguments from.  
                If specified, then other passed arguments are ignored.
        """
        self._SourceItems:list[SourceItem] = []
        
        if (root is None):
            
            pass
        
        else:

            # base fields.
            if (root.tag == 'sources'):
                for item in root:
                    self.append(SourceItem(root=item))
            else:
                source_item_list = root.find('sources')
                if source_item_list:
                    for item in root.find('sources'):
                        self.append(SourceItem(root=item))

            # sort items on Source property, descending order.
            if len(self._SourceItems) > 0:
                self._SourceItems.sort(key=lambda x: x.Source or "", reverse=False)


    def __getitem__(self, key) -> SourceItem:
        if isinstance(key, str):
            for item in self:
                if item.Source == key:
                    return item
        else:
            return self._SourceItems[key]


    def __iter__(self) -> Iterator:
        return iter(self._SourceItems)


    def __len__(self) -> int:
        return len(self._SourceItems)


    def __repr__(self) -> str:
        return self.ToString()


    def __str__(self) -> str:
        return self.ToString()


    @property
    def SourceItems(self) -> list[SourceItem]:
        """ 
        The list of `SourceItem` items. 
        """
        return self._SourceItems


    def append(self, value: SourceItem):
        """
        Append a new `SourceItem` item to the list.
        
        Args:
            value:
                The `SourceItem` object to append.
        """
        self._SourceItems.append(value)


    def ToDictionary(self, encoding:str='utf-8') -> dict:
        """ 
        Returns a dictionary representation of the class. 
        
        Args:
            encoding (str):
                encode type (e.g. 'utf-8', 'unicode', etc).  
                Default is 'utf-8'.
        """
        if encoding is None:
            encoding = 'utf-8'
        elm = self.ToElement()
        xml = tostring(elm, encoding=encoding).decode(encoding)
        
        # convert xml to dictionary.
        oDict:dict = xmltodict.parse(xml,
                                     encoding=encoding,
                                     process_namespaces=False)
        return oDict


    def ToElement(self) -> Element:
        """ 
        Returns an xmltree Element node representation of the class. 
        """
        elm = Element('sources')
        
        item:SourceItem
        for item in self:
            elm.append(item.ToElement())
        return elm


    def ToSourceArray(self, includeSourceAccount:bool=False) -> list[str]:
        """
        Returns an array of SourceItem (and optionally SourceAccount) strings.
        
        Args:
            includeSourceAccount (bool):
                True to include SourceAccount property values if present;
                otherwise, False to only return Source property values.
        
        If includeSourceAccount=True was specified and a SourceAccount value is present, 
        then it will return the Source entry as "Source:sourceAccount" as well as a 
        second entry of just the "Source" value; otherwise, just the "Source" value is 
        returned for the source item.
        """
        sourceList:list[str] = []
        
        # load list of supported sources.
        item:SourceItem
        for item in self:
            if (item is not None) and (len(item.Source) > 0):
                if (includeSourceAccount == False):
                    sourceList.append(item.Source)
                elif (item.SourceAccount is not None) and (len(item.SourceAccount) > 0):
                    sourceList.append("%s:%s" % (item.Source, item.SourceAccount))
                else:
                    sourceList.append(item.Source)
                        
        return sourceList

        
    def ToString(self, includeItems:bool=False) -> str:
        """
        Returns a displayable string representation of the class.
        
        Args:
            includeItems (bool):
                True to include all items in the list; otherwise False to only
                include the base list.
        """
        msg:str = 'SourceList:'
        msg = "%s (%d items)" % (msg, self.__len__())
        
        if includeItems == True:
            item:SourceItem
            for item in self:
                msg = "%s\n- %s" % (msg, item.ToString())
            
        return msg

==> bosesoundtouchapi-1.0.21-py3-none-any.whl: bosesoundtouchapi/models/sourcelist.py <==
# external package imports.
from typing import Iterator
from xml.etree.ElementTree import Element, tostring
import xmltodict

# our package imports.
from ..bstutils import export, _xmlFind, _xmlFindAttr
from .sourceitem import SourceItem

@export
class SourceList:
    """
    SoundTouch device SourceList configuration object.
       
    This class contains the attributes and sub-items that represent the
    sources configuration of the device.
    """

    def __init__(self, root:Element=None) -> None:
        """
        Initializes a new instance of the class.
        
        Args:
            root (Element):
                xmltree Element item to load arguments from.  
                If specified, then other passed arguments are ignored.
        """
        self._SourceItems:list[SourceItem] = []
        
        if (root is None):
            
            pass
        
        else:

            # base fields.
            if (root.tag == 'sources'):
                for item in root:
                    self.append(SourceItem(root=item))
            else:
                source_item_list = root.find('sources')
                if source_item_list:
                    for item in root.find('sources'):
                        self.append(SourceItem(root=item))

            # sort items on Source property, descending order.
            if len(self._SourceItems) > 0:
                self._SourceItems.sort(key=lambda x: x.Source or "", reverse=False)


    def __getitem__(self, key) -> SourceItem:
        if isinstance(key, str):
            for item in self:
                if item.Source == key:
                    return item
        else:
            return self._SourceItems[key]


    def __iter__(self) -> Iterator:
        return iter(self._SourceItems)


    def __len__(self) -> int:
        return len(self._SourceItems)


    def __repr__(self) -> str:
        return self.ToString()


    def __str__(self) -> str:
        return self.ToString()


    @property
    def SourceItems(self) -> list[SourceItem]:
        """ 
        The list of `SourceItem` items. 
        """
        return self._SourceItems


    def append(self, value: SourceItem):
        """
        Append a new `SourceItem` item to the list.
        
        Args:
            value:
                The `SourceItem` object to append.
        """
        self._SourceItems.append(value)


    def ToDictionary(self, encoding:str='utf-8') -> dict:
        """ 
        Returns a dictionary representation of the class. 
        
        Args:
            encoding (str):
                encode type (e.g. 'utf-8', 'unicode', etc).  
                Default is 'utf-8'.
        """
        if encoding is None:
            encoding = 'utf-8'
        elm = self.ToElement()
        xml = tostring(elm, encoding=encoding).decode(encoding)
        
        # convert xml to dictionary.
        oDict:dict = xmltodict.parse(xml,
                                     encoding=encoding,
                                     process_namespaces=False)
        return oDict


    def ToElement(self, isRequestBody:bool=False) -> Element:
        """ 
        Returns an xmltree Element node representation of the class. 

        Args:
            isRequestBody (bool):
                True if the element should only return attributes needed for a POST
                request body; otherwise, False to return all attributes.
        """
        elm = Element('sources')
        
        item:SourceItem
        for item in self:
            elm.append(item.ToElement())
        return elm


    def ToSourceArray(self, includeSourceAccount:bool=False) -> list[str]:
        """
        Returns an array of SourceItem (and optionally SourceAccount) strings.
        
        Args:
            includeSourceAccount (bool):
                True to include SourceAccount property values if present;
                otherwise, False to only return Source property values.
        
        If includeSourceAccount=True was specified and a SourceAccount value is present, 
        then it will return the Source entry as "Source:sourceAccount" as well as a 
        second entry of just the "Source" value; otherwise, just the "Source" value is 
        returned for the source item.
        """
        sourceList:list[str] = []
        
        # load list of supported sources.
        item:SourceItem
        for item in self:
            if (item is not None) and (len(item.Source) > 0):
                if (includeSourceAccount == False):
                    sourceList.append(item.Source)
                elif (item.SourceAccount is not None) and (len(item.SourceAccount) > 0):
                    sourceList.append("%s:%s" % (item.Source, item.SourceAccount))
                else:
                    sourceList.append(item.Source)
                        
        return sourceList

        
    def ToString(self, includeItems:bool=False) -> str:
        """
        Returns a displayable string representation of the class.
        
        Args:
            includeItems (bool):
                True to include all items in the list; otherwise False to only
                include the base list.
        """
        msg:str = 'SourceList:'
        msg = "%s (%d items)" % (msg, self.__len__())
        
        if includeItems == True:
            item:SourceItem
            for item in self:
                msg = "%s\n- %s" % (msg, item.ToString())
            
        return msg
